#include<stdio.h>
#include<ctype.h>
#include"resource.h"

main()
{
  char **sp;
  char *s;
  int c;

  sp = DefaultResources;

  while( *sp != NULL ){
    c = 0;
    s = *sp++;
    while( !isspace(*s) ){
      putchar( *s++ );
      c++;
    }
    c &= 8;

    while( c < 16 ){
      putchar('\t');
      c += 8;
    }
    printf(": ");
    while( *s == ':' || *s == ' ' || *s == '\t' )
      s++;
    puts(s);
  }
}
