static char *DefaultResources[] = {

"XYag*font              : -adobe-helvetica-*-r-*-*-14-*-*-*-*-*-*-*",
"XYag*justify           : center",
"XYag*Label.borderWidth : 0",
"XYag*horizDistance     : 8",
"XYag*vertDistance      : 8",

"XYag*ok.label      : O.K",
"XYag*cancel.label  : Cancel",

"XYag*dialog.icon        : Term",
"XYag*query.dialog.icon  : Excl",
"XYag*error.dialog.icon  : Excl",

"XYag*save.dialog*translations : #override <Key>Return : select_yes()",
"XYag*load.dialog*translations : #override <Key>Return : select_yes()",
"XYag*fps.dialog*translations  : #override <Key>Return : select_yes()",
"XYag*error.dialog*translations: #override <Key>Return : select_yes()",

"XYag*yes.label     : Yes",
"XYag*no.label      : No",

"XYag*save.dialog.label  : Save file:",
"XYag*load.dialog.label  : Load file:",
"XYag*fps.dialog.label   : Enter speed ( Frame per second )",
"XYag*query.dialog.label : File changed ; Save?",
           
"XYag*menu_b.label       : Global menu",
"XYag*menu_b.leftBitmap  : menu12",

"XYag*menu.width         : 128",

"XYag*menu.save.label     : Save file (^s)",
"XYag*menu.load.label     : Load file (^l)",
"XYag*menu.clear.label    : Clear all (^c)",
"XYag*menu.fps.label      : Set speed (^f)",
"XYag*menu.quit.label     : Quit (^q)", 

"XYag*console.Command.width     : 112",

"XYag*plabel.label      : Page",
"XYag*plabel.fromHoriz  : menu_b",

"XYag*number.fromHoriz : plabel",

"XYag*see_pp.label        : See prev",
"XYag*see_pp.fromHoriz    : number",
"XYag*see_pp.translations : #override \\n\
<Btn1Down>,<Btn1Up> : see(PREV)",

"XYag*see_np.label        : See next",
"XYag*see_np.fromHoriz    : see_pp",
"XYag*see_np.translations : #override \\n\
<Btn1Down>,<Btn1Up> : see(NEXT)",

"XYag*bar.fromVert     : plabel",
"XYag*bar.width        : 620",
"XYag*bar.height       : 16",
"XYag*bar.minimumThumb : 1",
"XYag*bar.orientation  : horizontal",

"XYag*add.label        : Add page (spc)",
"XYag*add.fromVert     : bar",
"XYag*add.translations : #override \\n\
<Btn1Down>,<Btn1Up> : page(ADD)unset()",

"XYag*ins.label        : Insert page (i)",
"XYag*ins.fromVert     : bar",
"XYag*ins.fromHoriz    : add",
"XYag*ins.translations : #override \\n\
<Btn1Down>,<Btn1Up> : page(INS)unset()",

"XYag*del.label        : Delete page (d)",
"XYag*del.fromVert     : bar",
"XYag*del.fromHoriz    : ins",
"XYag*del.translations : #override \\n\
<Btn1Down>,<Btn1Up> : page(DEL)unset()",

"XYag*cls.label        : Clear page (c)",
"XYag*cls.fromVert     : bar",
"XYag*cls.fromHoriz    : del",
"XYag*cls.translations : #override \\n\
<Btn1Down>,<Btn1Up> : page(CLEAR)unset()",

"XYag*play.label        : Play/Stop (p)",
"XYag*play.fromVert     : bar",
"XYag*play.fromHoriz    : cls",
"XYag*play.translations : #override \\n\
<Btn1Down>,<Btn1Up> : play(TOGGLE)unset()",

"XYag*scr.borderWidth   : 0",
"XYag*scr.fromVert      : console",

"XYag*yag.translations : #override \\n\
~Ctrl<Key>space   : page(ADD)   \\n\
~Ctrl<Key>Right   : page(+)     \\n\
~Ctrl<Key>Left    : page(-)     \\n\
~Ctrl<Key>i       : page(INS)   \\n\
~Ctrl<Key>d       : page(DEL)   \\n\
~Ctrl<Key>c       : page(CLEAR) \\n\
~Ctrl<Key>less    : page(FIRST) \\n\
~Ctrl<Key>greater : page(LAST)  \\n\
~Ctrl<Key>p       : play(TOGGLE)\\n\
\
Ctrl<Key>s   : file(SAVE)  \\n\
Ctrl<Key>l   : file(LOAD)  \\n\
Ctrl<Key>c   : file(CLEAR) \\n\
Ctrl<Key>f   : file(FPS)   \\n\
Ctrl<Key>q   : file(QUIT)  \\n\
",

"XYag*scr.translations	: #override \\n\
Shift<Btn1Down>   : point(START) \\n\
Shift<Btn1Motion> : point(NEXT)  \\n\
Shift<Btn1Up>     : point(CUT)   \\n\
\
None<Btn1Down>  : point(NEXT)point(START) \\n\
None<Btn2Down>  : point(BACK)             \\n\
None<Btn3Down>  : point(NEXT)point(CUT)   \\n\
\
<Motion> : point(MOVE) \\n\
",
NULL };
